# Copyright (c) 2011 John Stowers
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSES/GPL-3.0

import os
import os.path
import logging
import zipfile
import tempfile
import json

from gi.repository import Gtk
from gi.repository import GLib
from gtweak.tweakmodel import Tweak

from gtweak.utils import walk_directories, make_combo_list_with_default, extract_zip_file, get_resource_dirs
from gtweak.gshellwrapper import GnomeShellFactory
from gtweak.gtksettings import GtkSettingsManager
from gtweak.widgets import (TweakPreferencesPage, GSettingsTweakComboRow,TweakPreferencesGroup, GSettingsFileChooserButtonTweak, FileChooserButton, build_label_beside_widget)


_shell = GnomeShellFactory().get_shell()
_shell_loaded = _shell is not None

class GtkThemeSwitcher(GSettingsTweakComboRow):
    def __init__(self, **options):
        self._gtksettings3 = GtkSettingsManager('3.0')
        self._gtksettings4 = GtkSettingsManager('4.0')

        GSettingsTweakComboRow.__init__(self,
			_("Legacy Applications"),
            "org.gnome.desktop.interface",
            "gtk-theme",
            make_combo_list_with_default(self._get_valid_themes(), "Adwaita"),
            **options)


    def _get_valid_themes(self):
        """ Only shows themes that have variations for gtk3"""
        gtk_ver = Gtk.MINOR_VERSION
        if gtk_ver % 2: # Want even number
            gtk_ver += 1

        valid = ['Adwaita', 'HighContrast', 'HighContrastInverse']
        valid += walk_directories(get_resource_dirs("themes"), lambda d:
                    os.path.exists(os.path.join(d, "gtk-3.0", "gtk.css")) or \
                         os.path.exists(os.path.join(d, "gtk-3.{}".format(gtk_ver))))
        return set(valid)

    def _on_combo_changed(self, combo, _):
        item = combo.get_selected_item()
        if item:
            value = item.value
            self.settings.set_string(self.key_name, value)
        # Turn off Global Dark Theme when theme is changed.
        # https://bugzilla.gnome.org/783666
        try:
            self._gtksettings3.set_integer("gtk-application-prefer-dark-theme",
                                          0)
            self._gtksettings4.set_integer("gtk-application-prefer-dark-theme",
                                          0)
        except:
            self.notify_information(_("Error writing setting"))


class IconThemeSwitcher(GSettingsTweakComboRow):
    def __init__(self, **options):
        GSettingsTweakComboRow.__init__(self,
			_("Icons"),
			"org.gnome.desktop.interface",
            "icon-theme",
            make_combo_list_with_default(self._get_valid_icon_themes(), "Adwaita"),
            **options)

    def _get_valid_icon_themes(self):
        valid = walk_directories(get_resource_dirs("icons"), lambda d:
                    os.path.isdir(d) and \
			os.path.exists(os.path.join(d, "index.theme")))
        return set(valid)

class CursorThemeSwitcher(GSettingsTweakComboRow):
    def __init__(self, **options):
        GSettingsTweakComboRow.__init__(self,
			_("Cursor"),
            "org.gnome.desktop.interface",
            "cursor-theme",
            make_combo_list_with_default(self._get_valid_cursor_themes(), "Adwaita"),
            **options)

    def _get_valid_cursor_themes(self):
        valid = walk_directories(get_resource_dirs("icons"), lambda d:
                    os.path.isdir(d) and \
                        os.path.exists(os.path.join(d, "cursors")))
        return set(valid)

class ShellThemeTweak(GSettingsTweakComboRow):
    THEME_EXT_NAME = "user-theme@gnome-shell-extensions.gcampax.github.com"
    THEME_GSETTINGS_SCHEMA = "org.gnome.shell.extensions.user-theme"
    THEME_GSETTINGS_NAME = "name"
    THEME_GSETTINGS_DIR = os.path.join(GLib.get_user_data_dir(), "gnome-shell", "extensions",
                                       THEME_EXT_NAME, "schemas")
    LEGACY_THEME_DIR = os.path.join(GLib.get_home_dir(), ".themes")
    THEME_DIR = os.path.join(GLib.get_user_data_dir(), "themes")

    def __init__(self):
        #check the shell is running and the usertheme extension is present
        error = _("Unknown error")
        self._shell = _shell

        if self._shell is None:
            logging.warning("Shell not running", exc_info=True)
            error = _("Shell not running")
        else:
            try:
                extensions = self._shell.list_extensions()
                if ShellThemeTweak.THEME_EXT_NAME in extensions and extensions[ShellThemeTweak.THEME_EXT_NAME]["state"] == 1:
                    error = None

                else:
                    error = _("Shell user-theme extension not enabled")
            except Exception as e:
                logging.warning("Could not list shell extensions", exc_info=True)
                error = _("Could not list shell extensions")

        if error:
            valid = []
        else:
            #include both system, and user themes
            #note: the default theme lives in /system/data/dir/gnome-shell/theme
            #      and not themes/, so add it manually later
            dirs = [os.path.join(d, "themes") for d in GLib.get_system_data_dirs()]
            dirs += [ShellThemeTweak.THEME_DIR]
            dirs += [ShellThemeTweak.LEGACY_THEME_DIR]
            # add default theme directory since some alternative themes are installed here
            dirs += [os.path.join(d, "gnome-shell", "theme") for d in GLib.get_system_data_dirs()]

            valid = walk_directories(dirs, lambda d:
                    os.path.exists(os.path.join(d, "gnome-shell.css")) or \
                    (
                        os.path.exists(os.path.join(d, "gnome-shell")) and \
                        os.path.exists(os.path.join(d, "gnome-shell", "gnome-shell.css"))
                    ))
            #the default value to reset the shell is an empty string
            valid.extend( ("",) )
            valid = set(valid)
        
        # load the schema from the user installation of User Themes if it exists
        schema_dir = ShellThemeTweak.THEME_GSETTINGS_DIR if os.path.exists(ShellThemeTweak.THEME_GSETTINGS_DIR) else None

        # build a combo box with all the valid theme options
        GSettingsTweakComboRow.__init__(self,
		  title=_("Shell"),
          subtitle=error if error else None,
          schema_name=ShellThemeTweak.THEME_GSETTINGS_SCHEMA,
          schema_dir=schema_dir,
          key_name=ShellThemeTweak.THEME_GSETTINGS_NAME,
          key_options=make_combo_list_with_default(opts=list(valid), default="", default_text=_("Adwaita (default)")),
          loaded=_shell_loaded,
        )

class ShellThemeInstallerTweak(Gtk.Box, Tweak):
    def __init__(self, title, description=None, **options):
        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
        Tweak.__init__(self, title=title, description=description, **options)

        chooser = FileChooserButton(
                    _("Select a theme"),
                    ["application/zip"])
        chooser.connect("notify::file-uri", self._on_file_set)

        build_label_beside_widget(title, chooser, hbox=self)

        self.widget_for_size_group = None

    def _on_file_set(self, chooser: FileChooserButton, _pspec):
        f = chooser.get_absolute_path()

        if not f:
            return

        with zipfile.ZipFile(f, 'r') as z:
            try:
                fragment = ()
                theme_name = None
                for n in z.namelist():
                    if n.endswith("gnome-shell.css"):
                        fragment = n.split("/")[0:-1]
                    if n.endswith("gnome-shell/theme.json"):
                        logging.info("New style theme detected (theme.json)")
                        #new style theme - extract the name from the json file
                        tmp = tempfile.mkdtemp()
                        z.extract(n, tmp)
                        with open(os.path.join(tmp,n)) as f:
                            try:
                                theme_name = json.load(f)["shell-theme"]["name"]
                            except:
                                logging.warning("Invalid theme format", exc_info=True)

                if not fragment:
                    raise Exception("Could not find gnome-shell.css")

                if not theme_name:
                    logging.info("Old style theme detected (missing theme.json)")
                    #old style themes name was taken from the zip name
                    if fragment[0] == "theme" and len(fragment) == 1:
                        theme_name = os.path.basename(f)
                    else:
                        theme_name = fragment[0]

                theme_members_path = "/".join(fragment)

                ok, updated = extract_zip_file(
                                z,
                                theme_members_path,
                                os.path.join(ShellThemeTweak.THEME_DIR, theme_name, "gnome-shell"))

                if ok:
                    if updated:
                        self.notify_information(_("%s theme updated successfully") % theme_name)
                    else:
                        self.notify_information(_("%s theme installed successfully") % theme_name)
                else:
                    self.notify_information(_("Error installing theme"))


            except:
                # does not look like a valid theme
                self.notify_information(_("Invalid theme"))
                logging.warning("Error parsing theme zip", exc_info=True)

        # set button back to default state
        chooser.props.file_uri = None


TWEAK_GROUP = TweakPreferencesPage("appearance", _("Appearance"),
  TweakPreferencesGroup( _("Styles"), "title-styles",
    CursorThemeSwitcher(),
    IconThemeSwitcher(),
    ShellThemeTweak(),
    GtkThemeSwitcher(),
  ),
  # TODO: The current installer is brittle and the interaction doesn't make sense
  # (you select a file and then it is un-selected with notifications informing
  # you if it installed correctly)
  #
  #   ShellThemeInstallerTweak(
  #     title=_("Install custom shell theme"),
  #     description=_("Install custom or user themes for GNOME shell"),
  #   ),
  TweakPreferencesGroup(
    _("Background"), "title-backgrounds",
    GSettingsFileChooserButtonTweak(
      _("Default Image"),
      "org.gnome.desktop.background",
      "picture-uri",
      mimetypes=["application/xml", "image/png", "image/jpeg"],
    ),
    GSettingsFileChooserButtonTweak(
      _("Dark Style Image"),
      "org.gnome.desktop.background",
      "picture-uri-dark",
      mimetypes=["application/xml", "image/png", "image/jpeg"],
    ),
    GSettingsTweakComboRow(
      _("Adjustment"), "org.gnome.desktop.background", "picture-options"
    ),
   ),
)



DATA_DIR = "/usr/share"
APP_ID = "org.gnome.tweaks"
PKG_DATA_DIR = "/usr/share/gnome-tweaks"
GSETTINGS_SCHEMA_DIR = "/usr/share/glib-2.0/schemas"
TWEAK_DIR = "/usr/lib/python3.13/site-packages/gtweak/tweaks"
VERSION = "46.1"
LOCALE_DIR = "/usr/share/locale"



# Copyright (c) 2011 John Stowers
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSES/GPL-3.0

from gtweak.widgets import UI_BOX_HORIZONTAL_SPACING, UI_BOX_SPACING, _GSettingsTweak, GSettingsSwitchTweakValue, TweakPreferencesPage, GSettingsTweakComboRow, GSettingsTweakSwitchRow, TweakPreferencesGroup, TweaksCheckGroupActionRow, build_label_beside_widget

from gi.repository import Gtk


class Focus(TweaksCheckGroupActionRow):

    def __init__(self):
        name: str = _("Window Focus")
        desc: str = _("Click to Focus")
        TweaksCheckGroupActionRow.__init__(self, title=name, subtitle=desc, setting="org.gnome.desktop.wm.preferences", key_name="focus-mode", name="focus")

        self.add_row(
            key_name="click", title=_("Click to Focus"),
            subtitle=_(
                "Windows are focused when they are clicked."))

        self.add_row(
            key_name="sloppy", title=_("Focus on Hover"),
            subtitle=_(
                "Window is focused when hovered with the pointer. Windows remain focused when the desktop is hovered."))

        self.add_row(
            key_name="mouse", title=_("Focus Follows Mouse"),
            subtitle=_(
                "Window is focused when hovered with the pointer. Hovering the desktop removes focus "
                "from the previous window."))


depends_how = lambda x,kn: x.get_string(kn) in ("mouse", "sloppy")

class ShowWindowButtons(GSettingsSwitchTweakValue):

    def __init__(self, name, value, **options):
        self.value = value
        GSettingsSwitchTweakValue.__init__(self,
                                           name,
                                           "org.gnome.desktop.wm.preferences",
                                           "button-layout",
                                           **options)
    def get_active(self):
        return self.value in self.settings.get_string(self.key_name)

    def set_active(self, v):
        val = self.settings.get_string(self.key_name)
        (left, colon, right) = val.partition(":")

        if "close" in right:
            rsplit = right.split(",")
            rsplit = [x for x in rsplit if x in ['appmenu', 'minimize', 'maximize', 'close']]

            if v:
                rsplit.append(self.value)
            else:
                rsplit.remove(self.value)

            rsplit.sort(key=lambda x: ["appmenu", "minimize", "maximize", "close"].index(x))

            self.settings.set_string(self.key_name, left + colon + ",".join(rsplit))

        else:
            rsplit = left.split(",")
            rsplit = [x for x in rsplit if x in ['appmenu', 'minimize', 'maximize', 'close']]

            if v:
                rsplit.append(self.value)
            else:
                rsplit.remove(self.value)

            rsplit.sort(key=lambda x: ["close", "minimize", "maximize", "appmenu"].index(x))

            self.settings.set_string(self.key_name, ",".join(rsplit) + colon + right)

class PlaceWindowButtons(Gtk.Box, _GSettingsTweak):

    def __init__(self, **options):
        name = _("Placement")
        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL, margin_start=UI_BOX_HORIZONTAL_SPACING, margin_end=UI_BOX_HORIZONTAL_SPACING,
                         margin_top=UI_BOX_SPACING, margin_bottom=UI_BOX_SPACING,spacing=0)

        _GSettingsTweak.__init__(self,
                                 name,
                                 "org.gnome.desktop.wm.preferences",
                                 "button-layout",
                                 **options)

        box_btn = Gtk.Box()
        box_btn.set_homogeneous(True)
        box_btn.add_css_class("linked")

        # Translators: For RTL languages, this is the "Right" direction since the
        # interface is flipped
        btn1 = Gtk.ToggleButton.new_with_label(_("Left"))
        # Translators: For RTL languages, this is the "Left" direction since the
        # interface is flipped
        btn2 = Gtk.ToggleButton.new_with_label(_("Right"))
        btn2.set_group(btn1)

        val = self.settings.get_string(self.key_name)
        (left, colon, right) = val.partition(":")
        if "close" in right:
           btn2.set_active(True)
        elif "close" in left:
           btn1.set_active(True)
        btn2.connect("toggled", self.on_button_toggled)

        box_btn.append(btn1)
        box_btn.append(btn2)

        build_label_beside_widget(name, box_btn, hbox=self)

    def on_button_toggled(self, v):
        val = self.settings.get_string(self.key_name)
        (left, colon, right) = val.partition(":")

        if "close" in left:
            rsplit = left.split(",")
            rsplit = [x for x in rsplit if x in ['appmenu', 'minimize', 'maximize', 'close']]
            rsplit.sort(key=lambda x: ["appmenu", "minimize", "maximize", "close"].index(x))
            self.settings.set_string(self.key_name, right + colon + ",".join(rsplit))
        else:
            rsplit = right.split(",")
            rsplit = [x for x in rsplit if x in ['appmenu', 'minimize', 'maximize', 'close']]
            rsplit.sort(key=lambda x: ["close", "minimize", "maximize", "appmenu"].index(x))
            self.settings.set_string(self.key_name, ",".join(rsplit) + colon + left)


TWEAK_GROUP = TweakPreferencesPage(
    "window-management", _("Windows"),
    TweakPreferencesGroup(
        _("Titlebar Actions"), "title-titlebar-actions",
        GSettingsTweakComboRow(_("Double-Click"),"org.gnome.desktop.wm.preferences", "action-double-click-titlebar"),
        GSettingsTweakComboRow(_("Middle-Click"),"org.gnome.desktop.wm.preferences", "action-middle-click-titlebar"),
        GSettingsTweakComboRow(_("Secondary-Click"),"org.gnome.desktop.wm.preferences", "action-right-click-titlebar"),
    ), 
    TweakPreferencesGroup(_("Titlebar Buttons"), "title-theme",
        ShowWindowButtons(_("Maximize"), "maximize"),
        ShowWindowButtons(_("Minimize"), "minimize"),
        PlaceWindowButtons()
    ),
    TweakPreferencesGroup(
        _("Click Actions"), "title-window-behavior",
        GSettingsTweakSwitchRow(_("Attach Modal Dialogs"),"org.gnome.mutter", "attach-modal-dialogs",
                        desc=_("When on, modal dialog windows are attached to their parent windows, and cannot be moved.")),
        GSettingsTweakSwitchRow(_("Center New Windows"),"org.gnome.mutter", "center-new-windows"),
        GSettingsTweakComboRow(_("Window Action Key"),
                        "org.gnome.desktop.wm.preferences",
                        "mouse-button-modifier",
                        [("disabled", _("Disabled")), ("<Alt>", "Alt"), ("<Super>", "Super")]),
  
    
        GSettingsTweakSwitchRow(_("Resize with Secondary-Click"),"org.gnome.desktop.wm.preferences", "resize-with-right-button"),
     ),
 
 TweakPreferencesGroup(_("Window Focus"), "window-focus", 
                       Focus(),
                      GSettingsTweakSwitchRow(_("Raise Windows When Focused"),"org.gnome.desktop.wm.preferences", "auto-raise", depends_on=Focus(), depends_how=depends_how))
)    




# Copyright (c) 2011 John Stowers
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSES/GPL-3.0

import os.path
import json
import logging

from gi.repository import Gio
from gi.repository import GLib

import gtweak.utils
from gtweak.gsettings import GSettingsSetting


class _ShellProxy:
    def __init__(self):
        d = Gio.bus_get_sync(Gio.BusType.SESSION, None)

        self.proxy = Gio.DBusProxy.new_sync(
                            d, 0, None,
                            'org.gnome.Shell',
                            '/org/gnome/Shell',
                            'org.gnome.Shell',
                            None)

        self.proxy_extensions = Gio.DBusProxy.new_sync(
                            d, 0, None,
                            'org.gnome.Shell',
                            '/org/gnome/Shell',
                            'org.gnome.Shell.Extensions',
                            None)

        val = self.proxy.get_cached_property("Mode")
        if val is not None:
            self._mode = val.unpack()
        else:
            logging.warning("Error getting shell mode")
            self._mode = "user"

        val = self.proxy.get_cached_property("ShellVersion")
        if val is not None:
            self._version = val.unpack()
        else:
            logging.critical("Error getting shell version")
            self._version = "0.0.0"

    @property
    def mode(self):
        return self._mode

    @property
    def version(self):
        return self._version


class GnomeShell:

    def __init__(self, shellproxy, shellsettings):
        self._proxy = shellproxy
        self._settings = shellsettings

    def _execute_js(self, js):
        result, output = self._proxy.proxy.Eval('(s)', js)
        if not result:
            raise Exception(output)
        return output

    def restart(self):
        self._execute_js('global.reexec_self();')

    def reload_theme(self):
        self._execute_js('const Main = imports.ui.main; Main.loadTheme();')

    def list_extensions(self):
        return self._proxy.proxy_extensions.ListExtensions()

    @property
    def mode(self):
        return self._proxy.mode

    @property
    def version(self):
        return self._proxy.version


@gtweak.utils.singleton
class GnomeShellFactory:
    def __init__(self):
        try:
            proxy = _ShellProxy()
            settings = GSettingsSetting("org.gnome.shell")

            self.shell = GnomeShell(proxy, settings)

            logging.debug("Shell version: %s", str(proxy.version))
        except:
            self.shell = None
            logging.warn("Shell not installed or running")

    def get_shell(self):
        return self.shell


if __name__ == "__main__":
    gtweak.GSETTINGS_SCHEMA_DIR = "/usr/share/glib-2.0/schemas/"

    logging.basicConfig(format="%(levelname)-8s: %(message)s", level=logging.DEBUG)

    s = GnomeShellFactory().get_shell()
    print("Shell Version: %s" % s.version)
    print(s.list_extensions())

    print(s == GnomeShellFactory().get_shell())
